#imgviewer-index { background:#333; text-align:left; padding:3px 0 3px 9px; min-height: 24px; _height: 24px; } .imgZoomInCtrl { position:absolute; background:transparent url(http://us.i1.yimg.com/us.yimg.com/lib/smb/assets/hosting/yss/extensions/slideshow/zoom_sm_1.0.gif) no-repeat scroll 0%; width:26px; height:26px; z-index: 2; display:none; cursor:pointer; } #imgviewer-overlay { opacity: .8; filter: alpha(opacity=80); z-index: 2; background: #111; position: fixed; width: 100%; height: 100%; left: 0; top: 0; display: none; border: 0; } * html #imgviewer-overlay { border-top: 2000px solid #111; top: auto; bottom: 0; height: 0; } #imgviewer-stage { height: 80%; width: 90%; font-family: "Trebuchet MS", Arial; background: #111; color: #ddd; border: 2px solid #999; z-index: 3; left: 50%; top: 50%; position: fixed; text-align: center; display: none; padding: 9px; } #imgviewer-stage #imgviewer-closeButton, #imgviewer-slideshow li img { cursor:pointer; } #imgviewer-stage #imgviewer-closeButton { float:right; text-decoration:none; color: #fff; background:transparent url(http://us.i1.yimg.com/us.yimg.com/lib/smb/assets/hosting/yss/extensions/slideshow/close_2.2.gif) no-repeat scroll 0% 50%; display:inline; float:right; height:20px; margin:-25px 10px 0pt 0pt; width:20px; overflow: hidden; } #imgviewer-stage #imgviewer-closeButton:hover { background-position: 100% 50%; } #imageCell { position:relative; background: url(http://us.i1.yimg.com/us.yimg.com/lib/smb/assets/hosting/yss/extensions/slideshow/loading_1.0.gif) no-repeat scroll 50% 50%; } #imageCell img { position:relative; top:50%; } #imgviewer-stage #imgviewer-closeButton span {margin-left: -100px;} #captionCell { position:absolute; bottom:9px; left:0; _left:9px; width: 100%; } #imgviewer-caption { background:#333; padding:10px; height: 55px; overflow: auto; text-align:left; font-size:.85em; margin-right:9px; margin-left:9px; _margin:0; } #imgviewer-slideshow { position: absolute; bottom: 70px; _bottom: 85px; left: 50%; margin-left: -93px; width: 186px; height: 61px; z-index: 4; background: url(http://us.i1.yimg.com/us.yimg.com/lib/smb/assets/hosting/yss/extensions/slideshow/controls_ul_1.0.png) no-repeat; list-style: none; display:block; }  #imgviewer-slideshow.macBug {left:-1000em;}  #imgviewer-slideshow.macBug,x:-moz-any-link {left:50%; display:none;}  #imgviewer-slideshow.macBug,x:-moz-any-link,x:default {left:-1000em;display:block;}  #imgviewer-slideshow li { margin:0 1px; display:inline; list-style: none; } #imgviewer-slideshow li a { position: absolute; display: block; } #imgviewer-slideshow #slideshow-prev { left: 13px; top: 12px; } #imgviewer-slideshow #slideshow-prev, #imgviewer-slideshow #slideshow-prev img { width: 36px; height: 31px; _height: 36px; } #imgviewer-slideshow #slideshow-startstop { left: 70px; top: 7px; } #imgviewer-slideshow #slideshow-startstop, #imgviewer-slideshow #slideshow-startstop img { width: 46px; height: 41px; _height: 46px; } #imgviewer-slideshow .stopSlideShow { background:transparent url(http://us.i1.yimg.com/us.yimg.com/lib/smb/assets/hosting/yss/extensions/slideshow/stop_1.0.png) no-repeat 5px 3px; } #imgviewer-slideshow #slideshow-next { left: 139px; top: 13px; } #imgviewer-slideshow #slideshow-next, #imgviewer-slideshow #slideshow-next img { width: 33px; height: 30px; _height: 35px; } #imgviewer-stage #imgviewer-stageImage { background: #fff; } * html { background:url('about:blank') fixed; } * html body { height: 100%; } * html #imgviewer-overlay { position: absolute; top: expression(documentElement.scrollTop); } * html #imgviewer-stage { position: absolute; top: expression(documentElement.scrollTop + (this.parentNode.offsetHeight / 2)); } * html #imgviewer-slideshow { background: url(http://us.i1.yimg.com/us.yimg.com/lib/smb/assets/hosting/yss/extensions/slideshow/controls_ul_ie_1.0.gif) no-repeat; } #imgviewer-stage.stage-on { display:block; } #body.ap { z-index:1; position: absolute; width: 100%; } #imgviewer-slideshow .dimmed { opacity: .6; filter: alpha(opacity=60); cursor: default; }