/*
table {
    background: #eef;
    padding: 1em;
}

td {
    padding: 5;
}

#main {
    padding-top: 10em;
}
    */



/* from Yahoo! login */
p, form, fieldset, h1, h2, h3, h4, h5, h6{margin:0;padding:0}
fieldset{padding:10px 0}
img, fieldset{border:0}
legend{display:none}
label{font-weight:normal;cursor:pointer;cursor:hand}
.concatid{cursor:help}
a{color:#053799;text-decoration:none}
a:hover{text-decoration:underline}
.yregclb:after{content:".";display:block;font-size:0px;line-height:0;height:0;clear:both;visibility:hidden}
body{text-align:center;color:#333;margin:5px 0 0;padding:0}
#yregwp{text-align:left;margin:0 auto;width:750px}
#yregmst{position:relative;width:750px;margin:0 0 12px}
#yregmst .yregmstl{position:absolute;top:0;right:0}
#yregct{padding:0 0 10px}
#yreglg{float:right;width:240px;margin:0 0 20px} 
#yregtxt{margin:0 270px 0 20px}
#yregtctx #yregtxt {width:490px;margin:0 0 10px;} /* ctx template */
#yregft{text-align:center;font-size:77%;padding:10px 0 0;border-top:1px solid #777}
.yregbx{border:1px solid #656565;margin:0 0 10px;background-color:#f9f9f9} 
.yregbxi{margin:11px}
.top .yregbxi{margin-top:20px}
.ct, .ct .cl, .cb, .cb .cl{display:block;font-size:0;line-height:0;margin:0;height:4px;position:relative}
.ct, .cb{width:100%;right:-1px}
.ct .cl, .cb .cl{width:4px;left:-2px}
#yregtxt .yregbx{background-color:#e9f2d7;border-color:#87bb69}
#yregtxt .yregbx h2{font-size:109%}
#yregtxt .ct{background-image:url(https://a248.e.akamai.net/sec.yimg.com/i/reg/cr_grn_ne.gif)}
#yregtxt .ct .cl{background-image:url(https://a248.e.akamai.net/sec.yimg.com/i/reg/cr_grn_nw.gif)}
#yregtxt .cb{background-image:url(https://a248.e.akamai.net/sec.yimg.com/i/reg/cr_grn_se.gif)}
#yregtxt .cb .cl{background-image:url(https://a248.e.akamai.net/sec.yimg.com/i/reg/cr_grn_sw.gif)}
.flicker #yregtxt .yregbx {background-color:#e3f0ff;border-color:#bbdbfe} /*flicker blue */
.flicker #yregtxt .ct {background-image:url(https://a248.e.akamai.net/sec.yimg.com/i/reg/cr_flk_ne.gif)}
.flicker #yregtxt .ct .cl {background-image:url(https://a248.e.akamai.net/sec.yimg.com/i/reg/cr_flk_nw.gif)}
.flicker #yregtxt .cb {background-image:url(https://a248.e.akamai.net/sec.yimg.com/i/reg/cr_flk_se.gif)}
.flicker #yregtxt .cb .cl {background-image:url(https://a248.e.akamai.net/sec.yimg.com/i/reg/cr_flk_sw.gif)}
.flicker #yregtxt li h3 {color:#303030}
.podcast #yregtxt .yregbx {background-color:#A7D6E0;border-color:#50ADC1} /*podcast turquoise */
.podcast #yregtxt .ct {background-image:url(https://a248.e.akamai.net/sec.yimg.com/i/reg/cr_pdcst_ne.gif)}
.podcast #yregtxt .ct .cl {background-image:url(https://a248.e.akamai.net/sec.yimg.com/i/reg/cr_pdcst_nw.gif)}
.podcast #yregtxt .cb {background-image:url(https://a248.e.akamai.net/sec.yimg.com/i/reg/cr_pdcst_se.gif)}
.podcast #yregtxt .cb .cl {background-image:url(https://a248.e.akamai.net/sec.yimg.com/i/reg/cr_pdcst_sw.gif)}
.podcast #yregtxt li h3{color:#056D87}
.smallbz #yregtxt li h3{color:#39c}
#yreglg h1, #yreglg h2, #yreglg h3{font-size:114%;color:#333}
#yreglg h3{margin:0 0 4px}
#yreglg select, #yreglg input, #yreglg p, #yreglgtb td, #yreglgtb th{font-size:92%}
#yreglg p.yreglgsb input{font-size:107%}
#yreglg .yregnewssl, #yreglg .yreglgmd {font:77%/114% verdana}
#yreglg .yregnewssl{margin:-4px 0 10px}
#yreglg .yregnewssl span {color:#c00}
#yreglgtb, #yreglgtb td, #yreglgtb th, #yreglg .yreglgmd, .yreglgsu, .yreglgsb, .yreglgsut, .yregnewssl{text-align:right}
#yreglgtb {margin:0 0 8px;padding:0;width:215px}
#yreglgtb td, #yreglgtb th{width:215px}
#yreglgtb th{white-space:nowrap} 
#yreglgtb td select{width:124px}
#yreglgtb td{width:130px}
#yreglgtb td input{width:120px}
.yreglgsb{margin:12px 0 0}
.yreglgsut{margin:0 0 3px}
.yreglgsub{margin:15px 0 0 30px} /*image sign up*/
.yreglgsu a, .yreglgsut a{color:#039;font:bold 114%/114% arial;text-decoration:underline}
.yreglgsut a{font-size:122%;}
h2.yregdnt{margin:14px 0 0}
.yregdsilu{border-bottom:1px solid #999;margin:0 0 15px;padding:0 0 8px}
.yregdlisu, .yreglvpnu{border-top:1px solid #999;margin:15px 0 0;padding:14px 0 10px}
#yreglg .second h3{ font:bold 100% Arial;color:#333}
#yreglg .second p{font:77%/140% verdana}
#yregtxt ul{margin:0 0 10px;padding:0}
#yregtxt li{margin:18px 0 0;padding:0;list-style:none}
#yregtxt li h3{font:normal 122%/152% verdana;color:#496e03}
#yregtxt li cite{font-style:normal;line-height:129%}
.frontpage #yregtxt li{margin-top:15px}
.frontpage #yregtxt li h3{color:#666}
#yregtgen #yregct {margin-right:15px}
#yreglgtb td.yregyid {text-align:left;font-size:100%}
.yreglgdu {margin:0 0 5px}
.yreglgdu a {font:bold 107%/136% verdana;color:#039;text-decoration:underline}
#yregtgen #yregtxt h2, #yregpmtxt h3{font:bold 152%/152% arial;color:#333;margin:0} 
#yregpmtxt h3{margin:0 0 10px}
#yregptxt {padding:12px 0 0}
#yregtgen #yregtxt p.yregpti {color:#666;margin:0 0 2px;font:bold 100%/100% arial}
#yregptxt ul {margin:15px 0 0;padding:0;}
#yregptxt li {margin:0 0 1em 18px;list-style-type:disc;font:normal 100%/122% arial}
#yregpmtxt {margin:25px 0 0;padding:20px 0 0;border-top:1px solid #ccc}
#yregtgen #yregtxt{margin-bottom:20px}
#yregtgen #yregtxt #yreghtxt h3{margin:0;font:bold 107%/114% arial;color:#8C57A1}
#yregtgen #yregtxt li h3{font:bold 114%/122% arial}
#yregtgen #yregtxt p{margin:0 0 0.8em;line-height:129%}
#yregtgen #yregtxt .yregbpt li{margin:0 0 10px 4px;padding:0 0 10px 22px;background:url(https://a248.e.akamai.net/sec.yimg.com/i/reg/purple_arrow.gif) no-repeat 1px 4px}
.yregertxt{margin:15px 0 5px}
.yregertxt, #yreglg .yregertxt h2{color:#c00}
#yreglg .yregertxt h2{font-size:107%}
.yregertxt p{font:normal 92%/107% arial}
.yregab #yreglg{width:320px}
.yregab #yreglgtb{width:auto}
html>body #yreglgtb th{width:180px}
.yregab #yregabimg{padding:10px 0 0}
.yregab .yregrid{text-align:right}
#yregtgen.yregab #yregtxt{width:380px}
#yregtgen #yregtxt #yreghtxt h2{color:#7A067F}
.yregertxt{margin:15px 0 5px}
.yregertxt, #yreglg .yregertxt h2{color:#c00}
#yreglg .yregertxt h2{font-size:107%}
.yregertxt p{font:normal 92%/107% arial}
.yreglvpnu p{font:normal 92%/136% arial}
.yreglvpnu p a{font:bold 107%/139% arial;text-decoration:underline}
#yreglgtb td.yreglvid{text-align:left}
#yregfl{margin:0 20px 20px}
#yregfl h1{font-size:136%;line-height:167%}
#yregfl h2{font-size:107%}
#yregfl p{margin:0 0 1em;}
#yregfl li{margin-bottom:10px}
.appuser #yregtxt h2{font:normal 122%/122% verdana, sans-serif;color:#5B9B0F;margin-top:10px}
.appuser #yregtxt li{padding:0 20px 0}
.appuser #yregtxt h3{font-weight:bold;font-size:92%}
.appuser #yregtxt li.greenarrow h3{color:#548F0D}
.appuser #yregtxt li.grayarrow h3{color:#969696}
.appuser #yregtxt li.greenarrow{background: url(https://a248.e.akamai.net/sec.yimg.com/i/reg/green_arrow.gif) no-repeat 0 2px}
.appuser #yregtxt li.grayarrow{background: url(https://a248.e.akamai.net/sec.yimg.com/i/reg/gray_arrow.gif) no-repeat 0 2px}
.appuser #yregtxt ul li cite{color:#666}
#yregtgen #yregtxt #promoImg{float:right;margin:20px 10px 0}
#yregtgen #yregtxt #promoFooterTxt{font-size:85%}

.ct{background:transparent url(cr_gg_ne.gif) no-repeat top right;top:-1px}
.ct .cl{background:transparent url(cr_gg_nw.gif) no-repeat top left}
.cb{background:transparent url(https://a248.e.akamai.net/sec.yimg.com/i/reg/cr_gg_se.gif) no-repeat bottom right;bottom:-1px}
.cb .cl{background:transparent url(https://login.yahoo.com/i/reg/cr_gg_sw.gif) no-repeat bottom left}
.clear{clear:both}
#yreglg select, #yreglg input, #yreglg p, #yreglgtb td, #yreglgtb th{font-size:93%}
div.yregdsilu h2.yregdnt, div.yregdsilu p.yregsueasy{width:110px}
/*popup template css */
#yregtpopup #yregtxt {width:260px;margin:0 0 10px} /* popup template */
#yregtpopup #yregwp, #yregtpopup #yregmst{width:525px}
#yregtpopup #yregmst{margin-bottom:5px}
#yregtpopup #yreglg{margin-bottom:0}
#yregtpopup #yregft{padding-top:5px}

#yregtgen #yregtxt h2, #yregtpopup #yregtxt h2, #yregpmtxt h3{font:bold 152%/152% arial;color:#333;margin:0}
#yregtgen #yregtxt p.yregpti, #yregtpopup #yregtxt p.yregpti {color:#666;margin:0 0 2px;font:bold 100%/100% arial}
#yregtgen #yregtxt, #yregtpopup #yregtxt{margin-bottom:20px}
#yregtgen #yregtxt #yreghtxt h3, #yregtpopup #yregtxt #yreghtxt h3{margin:0;font:bold 107%/114% arial;color:#8C57A1}
#yregtgen #yregtxt li h3, #yregtpopup #yregtxt li h3{font:bold 114%/122% arial}
#yregtgen #yregtxt p, #yregtpopup #yregtxt p{margin:0 0 0.8em;line-height:129%}
#yregtgen #yregtxt .yregbpt li, #yregtpopup #yregtxt .yregbpt li{margin:0 0 10px 4px;padding:0 0 10px 22px;background:url(https://a248.e.akamai.net/sec.yimg.com/i/reg/purple_arrow.gif) no-repeat 1px 4px}

#yregtgen.yregab #yregtxt{width:auto;}
#yregtgen.yregab #yreghtxt{margin-right:60px;}
#yregtpopup.yregab #yregtxt{width:180px}
#yregtgen #yregtxt #yreghtxt h2, #yregtpopup #yregtxt #yreghtxt h2{color:#7A067F}
.yregbx{z-index:3;margin-right:0!important}
.flicker h3 span {color:#ff0084;font-weight:bold}
.flicker h3 a {text-decoration:underline;}

/* persistency message right above "sign in" bottom */
em.nwred a {font-style: normal; font-size: 85%; vertical-align:5px;}
.kmsibold {font-weight:bold; font-size: 114%;}
input#persistent {margin-bottom: -0em;}
.subperstxt {line-height:1.75em;}
.subperstxt2 {margin: 0 0 0 2em; display:block;}
/* #yregft p.yregfb { font-size:120%; padding-bottom: 5px; padding-up: 5px} */

.yreglgsb{margin-top:0}

#yregwp #yregct #yreglg .yregbxi #yreglgmd {margin-top:1.75em}
body#yregtgen fieldset {margin-bottom:2.5em}
#yreglgtb tr {width:17.92em}
/* p#sigcopys {text-align: left; font-size: 85%; float: right; padding: .4em; margin: .6em .4em 1em 0; border-bottom: 1px dotted #9D9C9D; border-top: 1px dotted #9D9C9D;}
 * */
p#sigcopys {text-align: left; font-size: 85%; padding: .4em; margin: .6em .0em 1em 0; border-bottom: 1px dotted #9D9C9D; border-top: 1px dotted #9D9C9D;}
#sigcopys label{display:block; margin:-1.5em 0 0 2em;}


#yregtgen #yregct {margin-right: 0px;}
#yregtgen #yregtxt { margin-left: 5px; margin-right: 245px }

/*anti phish badge */
.top {position:relative}
#antiphish{position:absolute;right:5px;top:5px;}
#antiphish.dogear{right:0px;top:0px;}
#antiphish a {font-size:92%;}
img.picture {border:2px solid}

/* badge backgrounds */
.badge{background-color:#f9f9f9; background-repeat:no-repeat; background-position:top right;}
.badge #yreglgtb {margin-top:18px;} /* increased badge size */


/* popup code... */
#security {display:none;position:absolute;top:-15px;left:-85px;z-index:1000;background-color:#a5a5a5;}
#security.noimage {left:-76px;top:-10px}
#securityi{position:relative;z-index:1;right:1px;bottom:1px;padding:11px;width:219px;background-color:#fff;border:1px solid #636363;}
#knob{position:absolute;top:30px;right:-10px;width:10px;height:18px;background:url(https://a248.e.akamai.net/sec.yimg.com/i/reg/sideknob.png) no-repeat top left}
.noimage #knob{top:22px}
#security p, #security ul li{font:77%/107% verdana;}
#security p a {text-decoration:underline;}
#security p{padding-bottom:5px;}
#security ul{margin:5px 0 0;padding:0 5px 0 0;text-align:right;list-style:none;}
#security ul li{margin:0;padding:0 0 2px;}

/* help text updates... */
#yregtgen #yregtxt .yregbpt li ul{margin:10px 0 0;padding:0 0 0 15px;}
#yregtgen #yregtxt .yregbpt li ul li{background:none;list-style:disc;margin:0 0 5px 0;padding:0;}
#yreghtxt ul{margin-left:0}
#yreghtxt ul.inlineHeaders li h3{display:inline;}
/* remove top margin on li ul */
.addressbar {display:block;margin:1em 0 1em 0}
.mono{font-family: courier new, courier, monospace;color:#000;font-weight:bold}


#rcta {width:99%; border:1px solid #898989; margin-top:10px; background-image:url(https://a248.e.akamai.net/sec.yimg.com/i/reg/gradient.png); background-repeat:repeat-x; background-color:#fde37c}
.ct {background:url(upper-right.gif) no-repeat scroll right top; top:-1px}
.ct .cl {background:url(upper-left.gif) no-repeat scroll left top;}
#rcta .key {width:40px; height:40px; border:1px solid #666666; background-image:url(https://a248.e.akamai.net/sec.yimg.com/i/reg/key.png); background-repeat: no-repeat; float:left; margin-top:1px}
#rcta .txt {margin-left:48px}
.cb {background:url(lower-right.gif) no-repeat scroll right bottom; bottom:-1px}
.cb .cl {background:url(lower-left.gif) no-repeat scroll left bottom;}
#rcta .ctact {margin:4px 10px;min-height:44px}
#rcta .txt .qs {font:normal bold 92% arial, Helvetica, sans-serif; color:#000; text-decoration:none}
#rcta .txt .sl {font:normal normal 100% arial, Helvetica, sans-serif; color:#000; text-decoration:none}
#rcta .txt .why {font:normal normal 85% arial, Helvetica, sans-serif;}
#rcta .txt .sltxt {line-height:0.9em}
.sltxt a {line-height:0.5em; font-size:85%}
.sltxt .why a{font-size:100%}
.yregertxt { margin-top: 25px }
.yreglgsut { margin-top: 15px }

#yreglgtb, #yreglgtb th {text-align: left; width: 100%;}
#yreglgtb td { width:179px;  text-align: left; padding: 0 0 16px 0}
#yreglgtb td input{ width:179px  } 
.dbidTip {padding: 3px 0 0 0;  font-size:85%}  

#yregtgen #yregtxt p.yregpti, #yregtpopup #yregtxt p.yregpti {login?.d...&.intl=us (line 39)
    -x-system-font:none;
    color:#666666;
    font-family:arial;
    font-size:100%;
    font-size-adjust:none;
    font-stretch:normal;
    font-style:normal;
    font-variant:normal;
    font-weight:bold;
    line-height:100%;
    margin:0 0 2px;
}

.page-title {
    text-align: left;
    font-size: 14pt;
    color: #c00;
    padding-left: 12px;
    font-weight: bold;
}

#register-link, #login-button {
    float: right;
    margin-right: 2em
    font-size: 14pt;
    padding: 2px;
}

#yreglg p.yreglgsb input {
    font-size:107%;
}

div#wait-message {
    color: red !important;
    background: white !important;
    font-size: 18px !important;
    float: right;
    position: fixed;
    top: 2px;
    right: 200px;
    padding: 4px;
    display: block;
}

ul#def-list li {
    list-style-image: none !important;
    list-style-type:square !important;
    list-style-position: inside !important;
}

ul#def-list {
    padding-left: 20px !important;
    display:block;
    list-style-type:square !important;
}

