/* Base */
@import url(themes/common/base-weblog.css);

/* Tip Jar */
@import url(themes/common/tipjar.css);

/* Portal */


/* Theme */
@import url(themes/lilia/theme-bluecrush.css);

/* Custom */
body
{
        font-family: 'Corbel', 'Cambria', 'trebuchet ms', helvetica, arial, sans-serif;
        font-size: 15px;
        color: black;
}

tt
{
        font-family: 'Consolas', 'Courier New', 'FreeMono', monospace;
}

h1, h2, h3, h4, h5, h6
{
        font-family: 'Cambria', 'Corbel', 'Candara', 'trebuchet ms', helvetica, arial, sans-serif;
        font-weight: bold;
}

a { color: #009; text-decoration: underline; }
a:visited { color: #306 }

.entry h2 {
	color: #930;
	font-size: 18px;
	border-bottom: 1px dotted #930;
	margin-bottom: 15px;
	font-weight: bold;
	}

h3 {
	color: #000;
	font-size: 15px;
	margin-bottom: 10px;
	}


